searchData={"content_type":"text/markdown","items":[{"doc":"`Actors` It's the client API for the Spawn actors.\nThrough this module we interact with the actors by creating,\ninvoking or configuring them.","ref":"Actors.html","title":"Actors","type":"module"},{"doc":"","ref":"Actors.html#get_state/1","title":"Actors.get_state/1","type":"function"},{"doc":"Makes a request to an actor.\n\n  * `request` - The InvocationRequest\n  * `opts` - The options to Invoke Actors\n##","ref":"Actors.html#invoke/2","title":"Actors.invoke/2","type":"function"},{"doc":"Makes a request to an actor using Nats broker.\n\n  * `request` - The InvocationRequest\n  * `opts` - The options to Invoke Actors\n##","ref":"Actors.html#invoke_with_nats/2","title":"Actors.invoke_with_nats/2","type":"function"},{"doc":"Registers all actors defined in HostActor.\n\n  * `registration` - The RegistrationRequest\n  * `opts` - The options to create Actors\n##","ref":"Actors.html#register/2","title":"Actors.register/2","type":"function"},{"doc":"Spawn actors defined in HostActor.\n\n  * `registration` - The SpawnRequest\n  * `opts` - The options to create Actors\n\nspawn_actor must be used when you want to create a concrete instance of an actor\npreviously registered as unnamed.\nThat is, when an Actorid is associated with an actor of unnamed type.\nThis function only registers the metadata of the new actor, not activating it.\nThis will occur when the sprite is first invoked.\n##","ref":"Actors.html#spawn_actor/2","title":"Actors.spawn_actor/2","type":"function"},{"doc":"An Elixir module representing a GenStage consumer responsible for handling\nevents initiated by `CallerProducer` and interacting with actors in the system.","ref":"Actors.Actor.CallerConsumer.html","title":"Actors.Actor.CallerConsumer","type":"module"},{"doc":"Performs the action of looking up or creating an actor based on the given parameters.\n\nThis function is responsible for looking up or creating an actor based on the specified actor\nfully-qualified domain name (FQDN). It incorporates distributed tracing with OpenTelemetry\nto capture relevant events and attributes during the lookup or creation process.","ref":"Actors.Actor.CallerConsumer.html#do_lookup_action/4","title":"Actors.Actor.CallerConsumer.do_lookup_action/4","type":"function"},{"doc":"- `system_name` (String): The name of the actor system.\n- `actor_fqdn` (tuple): A tuple representing the fully-qualified domain name (FQDN) of the actor.\n- `system` (%ActorSystem{}): The actor system.\n- `action_fun` (function): The function to be invoked once the actor is looked up or created.\n  It receives the actor reference and actor reference ID as parameters.","ref":"Actors.Actor.CallerConsumer.html#do_lookup_action/4-parameters","title":"Parameters - Actors.Actor.CallerConsumer.do_lookup_action/4","type":"function"},{"doc":"The function sets up a span with the name \"actor-lookup\" to trace the lookup or creation\nprocess. It adds relevant attributes, such as the actor FQDN, to the tracing context.","ref":"Actors.Actor.CallerConsumer.html#do_lookup_action/4-tracing-context","title":"Tracing Context - Actors.Actor.CallerConsumer.do_lookup_action/4","type":"function"},{"doc":"The function incorporates a retry mechanism with backoff, randomization, and timeout\nto handle potential errors during the lookup or creation process.","ref":"Actors.Actor.CallerConsumer.html#do_lookup_action/4-retry-mechanism","title":"Retry Mechanism - Actors.Actor.CallerConsumer.do_lookup_action/4","type":"function"},{"doc":"The result of the `action_fun` function or an error tuple in case of failure.","ref":"Actors.Actor.CallerConsumer.html#do_lookup_action/4-returns","title":"Returns - Actors.Actor.CallerConsumer.do_lookup_action/4","type":"function"},{"doc":"In case of errors during the lookup or creation process, appropriate logging and tracing events\nare added, and the error is returned as part of the result tuple.","ref":"Actors.Actor.CallerConsumer.html#do_lookup_action/4-error-handling","title":"Error Handling - Actors.Actor.CallerConsumer.do_lookup_action/4","type":"function"},{"doc":"Gets the state of the specified actor.\n\nThis function attempts to retrieve the state of the actor identified by the given\n`ActorId`. It uses an exponential backoff strategy for retrying in case of errors\nand logs any failures.","ref":"Actors.Actor.CallerConsumer.html#get_state/1","title":"Actors.Actor.CallerConsumer.get_state/1","type":"function"},{"doc":"- `id` (%ActorId): The unique identifier of the actor.","ref":"Actors.Actor.CallerConsumer.html#get_state/1-parameters","title":"Parameters - Actors.Actor.CallerConsumer.get_state/1","type":"function"},{"doc":"The state of the actor if successful, otherwise an error is raised.","ref":"Actors.Actor.CallerConsumer.html#get_state/1-returns","title":"Returns - Actors.Actor.CallerConsumer.get_state/1","type":"function"},{"doc":"The function utilizes an exponential backoff strategy with randomized delays and\na maximum expiry time of 30,000 milliseconds.","ref":"Actors.Actor.CallerConsumer.html#get_state/1-retry-strategy","title":"Retry Strategy - Actors.Actor.CallerConsumer.get_state/1","type":"function"},{"doc":"The function handles errors such as `:error`, `:exit`, `:noproc`, `:erpc`,\n`:noconnection`, and `:timeout`. It also rescues `ErlangError` exceptions and logs\ndetailed error messages.","ref":"Actors.Actor.CallerConsumer.html#get_state/1-errors","title":"Errors - Actors.Actor.CallerConsumer.get_state/1","type":"function"},{"doc":"Handles incoming events from the `CallerProducer` GenStage.\n\nDispatches events to the appropriate functions for further processing.","ref":"Actors.Actor.CallerConsumer.html#handle_events/3","title":"Actors.Actor.CallerConsumer.handle_events/3","type":"function"},{"doc":"Initializes the GenStage consumer.\n\nIt subscribes to the `CallerProducer` GenStage with specified backpressure values.","ref":"Actors.Actor.CallerConsumer.html#init/1","title":"Actors.Actor.CallerConsumer.init/1","type":"function"},{"doc":"Invokes an actor action with distributed tracing using OpenTelemetry.\n\nThis function performs an actor action invocation, incorporating distributed tracing\nwith OpenTelemetry. It sets up the tracing context, adds relevant attributes,\nand handles asynchronous and synchronous invocations.","ref":"Actors.Actor.CallerConsumer.html#invoke_with_span/2","title":"Actors.Actor.CallerConsumer.invoke_with_span/2","type":"function"},{"doc":"- `request` (%InvocationRequest): The request containing information about the invocation.\n- `opts` (Keyword.t): Additional options for the invocation. Defaults to an empty keyword list.","ref":"Actors.Actor.CallerConsumer.html#invoke_with_span/2-parameters","title":"Parameters - Actors.Actor.CallerConsumer.invoke_with_span/2","type":"function"},{"doc":"A tuple containing the status and the result of the invocation.\nIf the invocation is asynchronous, it returns `{:ok, :async}`.","ref":"Actors.Actor.CallerConsumer.html#invoke_with_span/2-returns","title":"Returns - Actors.Actor.CallerConsumer.invoke_with_span/2","type":"function"},{"doc":"The function sets up the tracing context and adds attributes related to the invocation.\nIt uses OpenTelemetry to trace the client invoke with the kind set to `:client`.","ref":"Actors.Actor.CallerConsumer.html#invoke_with_span/2-tracing-context","title":"Tracing Context - Actors.Actor.CallerConsumer.invoke_with_span/2","type":"function"},{"doc":"The function incorporates a retry mechanism with backoff, randomization, and timeout\nto handle potential errors during the invocation.","ref":"Actors.Actor.CallerConsumer.html#invoke_with_span/2-retry-mechanism","title":"Retry Mechanism - Actors.Actor.CallerConsumer.invoke_with_span/2","type":"function"},{"doc":"In case of errors during the invocation, appropriate logging and tracing events are added,\nand the error is re-raised with a stack trace.","ref":"Actors.Actor.CallerConsumer.html#invoke_with_span/2-error-handling","title":"Error Handling - Actors.Actor.CallerConsumer.invoke_with_span/2","type":"function"},{"doc":"Registers actors in the system based on the provided registration request.\n\nHandles registration requests and ensures actors are properly registered.","ref":"Actors.Actor.CallerConsumer.html#register/2","title":"Actors.Actor.CallerConsumer.register/2","type":"function"},{"doc":"Spawns an actor or a group of actors based on the provided `SpawnRequest`.\n\nThis function is responsible for spawning actors based on the specified `SpawnRequest`.\nIt retrieves the hosts associated with the provided actor IDs and registers the actors.\nAdditionally, it handles cases where the system is in the process of draining or stopping.","ref":"Actors.Actor.CallerConsumer.html#spawn_actor/2","title":"Actors.Actor.CallerConsumer.spawn_actor/2","type":"function"},{"doc":"- `spawn` (%SpawnRequest): The request containing information about the actors to spawn.\n- `opts` (Keyword.t): Additional options for spawning the actors. Defaults to an empty keyword list.","ref":"Actors.Actor.CallerConsumer.html#spawn_actor/2-parameters","title":"Parameters - Actors.Actor.CallerConsumer.spawn_actor/2","type":"function"},{"doc":"If successful, it returns `{:ok, %SpawnResponse{status: %RequestStatus{status: :OK, message: \"Accepted\"}}}`.\nOtherwise, an error is raised.","ref":"Actors.Actor.CallerConsumer.html#spawn_actor/2-returns","title":"Returns - Actors.Actor.CallerConsumer.spawn_actor/2","type":"function"},{"doc":"- Retrieves actor hosts based on actor IDs from the `ActorRegistry`.\n- Filters the hosts based on the system's graceful shutdown status.\n- Registers the selected hosts in the `ActorRegistry`.\n- Returns a success response.","ref":"Actors.Actor.CallerConsumer.html#spawn_actor/2-actor-spawning-process","title":"Actor Spawning Process - Actors.Actor.CallerConsumer.spawn_actor/2","type":"function"},{"doc":"- Raises an `ArgumentError` if attempting to spawn an unnamed actor that has not been registered before.","ref":"Actors.Actor.CallerConsumer.html#spawn_actor/2-errors","title":"Errors - Actors.Actor.CallerConsumer.spawn_actor/2","type":"function"},{"doc":"Starts the consumer process and subscribes to the `CallerProducer` GenStage.","ref":"Actors.Actor.CallerConsumer.html#start_link/1","title":"Actors.Actor.CallerConsumer.start_link/1","type":"function"},{"doc":"Tries to reactivate an actor.\n\nReactivation is attempted by looking up the actor in the registry\nor creating a new actor if not found.","ref":"Actors.Actor.CallerConsumer.html#try_reactivate_actor/3","title":"Actors.Actor.CallerConsumer.try_reactivate_actor/3","type":"function"},{"doc":"# Actors.Actor.CallerProducer\nThis module defines a GenStage producer responsible for managing actor-related\noperations such as retrieving actor state, actor registration, actor spawning, and actor invocation.","ref":"Actors.Actor.CallerProducer.html","title":"Actors.Actor.CallerProducer","type":"module"},{"doc":"The client API provides functions for interacting with actors.\nThese functions can be used to initiate operations such as getting actor state,\nregistering actors, spawning new actors, and invoking actors.\n\n#","ref":"Actors.Actor.CallerProducer.html#module-client-api","title":"Client API - Actors.Actor.CallerProducer","type":"module"},{"doc":"- [`start_link/1`](#start_link-1): Starts the CallerProducer process.\n- [`get_state/2`](#get_state-2): Retrieves the state of a specified actor.\n- [`register/2`](#register-2): Registers an actor with the specified registration request.\n- [`spawn_actor/2`](#spawn_actor-2): Spawns an actor based on the specified spawn request.\n- [`invoke/2`](#invoke-2): Invokes an actor with the specified invocation request.\n- [`enqueue/1`](#enqueue-1): Enqueues an event for processing.","ref":"Actors.Actor.CallerProducer.html#module-functions","title":"Functions - Actors.Actor.CallerProducer","type":"module"},{"doc":"The server API includes functions that handle the GenStage behavior.\nThese functions manage the state and processing of events.\n\n#","ref":"Actors.Actor.CallerProducer.html#module-server-api","title":"Server API - Actors.Actor.CallerProducer","type":"module"},{"doc":"- [`init/1`](#init-1): Initializes the GenStage process.\n- [`handle_demand/2`](#handle_demand-2): Handles demand requests from consumers.\n- [`handle_call/3`](#handle_call-3): Handles call requests from consumers.\n- [`handle_cast/2`](#handle_cast-2): Handles cast requests from consumers.","ref":"Actors.Actor.CallerProducer.html#module-functions","title":"Functions - Actors.Actor.CallerProducer","type":"module"},{"doc":"To interact with this module, use the client API functions provided.\nThese functions handle backpressure if enabled in the configuration.","ref":"Actors.Actor.CallerProducer.html#module-usage","title":"Usage - Actors.Actor.CallerProducer","type":"module"},{"doc":"Enqueues an event for processing.","ref":"Actors.Actor.CallerProducer.html#enqueue/1","title":"Actors.Actor.CallerProducer.enqueue/1","type":"function"},{"doc":"- `event` (any): The event to be enqueued.","ref":"Actors.Actor.CallerProducer.html#enqueue/1-parameters","title":"Parameters - Actors.Actor.CallerProducer.enqueue/1","type":"function"},{"doc":"Retrieves the state of a specified actor.","ref":"Actors.Actor.CallerProducer.html#get_state/2","title":"Actors.Actor.CallerProducer.get_state/2","type":"function"},{"doc":"- `actor_id` (ActorId.t()): The ID of the actor.\n- `opts` (any): Additional options.","ref":"Actors.Actor.CallerProducer.html#get_state/2-parameters","title":"Parameters - Actors.Actor.CallerProducer.get_state/2","type":"function"},{"doc":"- `{:ok, state}`: If the state is successfully retrieved.\n- `{:error, reason}`: If an error occurs during the operation.","ref":"Actors.Actor.CallerProducer.html#get_state/2-returns","title":"Returns - Actors.Actor.CallerProducer.get_state/2","type":"function"},{"doc":"Invokes an actor with the specified invocation request.","ref":"Actors.Actor.CallerProducer.html#invoke/2","title":"Actors.Actor.CallerProducer.invoke/2","type":"function"},{"doc":"- `request` (InvocationRequest.t()): The invocation request.\n- `opts` (any): Additional options.","ref":"Actors.Actor.CallerProducer.html#invoke/2-parameters","title":"Parameters - Actors.Actor.CallerProducer.invoke/2","type":"function"},{"doc":"- `{:ok, :async}`: If the invocation is asynchronous.\n- `{:ok, result}`: If the invocation is successful.\n- `{:error, reason}`: If an error occurs during the invocation.","ref":"Actors.Actor.CallerProducer.html#invoke/2-returns","title":"Returns - Actors.Actor.CallerProducer.invoke/2","type":"function"},{"doc":"Registers an actor with the specified registration request.","ref":"Actors.Actor.CallerProducer.html#register/2","title":"Actors.Actor.CallerProducer.register/2","type":"function"},{"doc":"- `registration` (RegistrationRequest.t()): The registration request.\n- `opts` (any): Additional options.","ref":"Actors.Actor.CallerProducer.html#register/2-parameters","title":"Parameters - Actors.Actor.CallerProducer.register/2","type":"function"},{"doc":"- `{:ok, response}`: If the actor is successfully registered.\n- `{:error, response}`: If an error occurs during the registration.","ref":"Actors.Actor.CallerProducer.html#register/2-returns","title":"Returns - Actors.Actor.CallerProducer.register/2","type":"function"},{"doc":"Spawns an actor based on the specified spawn request.","ref":"Actors.Actor.CallerProducer.html#spawn_actor/2","title":"Actors.Actor.CallerProducer.spawn_actor/2","type":"function"},{"doc":"- `spawn_req` (SpawnRequest.t()): The spawn request.\n- `opts` (any): Additional options.","ref":"Actors.Actor.CallerProducer.html#spawn_actor/2-parameters","title":"Parameters - Actors.Actor.CallerProducer.spawn_actor/2","type":"function"},{"doc":"- `{:ok, response}`: If the actor is successfully spawned.\n- `{:error, response}`: If an error occurs during the spawning.","ref":"Actors.Actor.CallerProducer.html#spawn_actor/2-returns","title":"Returns - Actors.Actor.CallerProducer.spawn_actor/2","type":"function"},{"doc":"Starts the CallerProducer process.","ref":"Actors.Actor.CallerProducer.html#start_link/1","title":"Actors.Actor.CallerProducer.start_link/1","type":"function"},{"doc":"- `state` (any): Initial state for the process.","ref":"Actors.Actor.CallerProducer.html#start_link/1-parameters","title":"Parameters - Actors.Actor.CallerProducer.start_link/1","type":"function"},{"doc":"- `:ignore`: If the process is already running.\n- `{:error, reason}`: If an error occurs during process initialization.\n- `{:ok, pid}`: If the process starts successfully.","ref":"Actors.Actor.CallerProducer.html#start_link/1-returns","title":"Returns - Actors.Actor.CallerProducer.start_link/1","type":"function"},{"doc":"Manages the lifecycle of the Host Actor through the `Entity` module.\n\nThe `Entity` module provides a GenServer-based implementation for controlling the\nlifecycle of actors, handling various actions, and interacting with the underlying\nactor system.","ref":"Actors.Actor.Entity.html","title":"Actors.Actor.Entity","type":"module"},{"doc":"The module implements GenServer behavior with transient restart semantics.\n\n- **Initialization:** The module initializes the actor state and handles the loading of persisted states.\n\n- **State Handling:** Manages the lifecycle of the actor, including state transitions, initialization actions, and periodic snapshots.\n\n- **Interaction:** Exposes client APIs for retrieving actor state, synchronously invoking actions, and asynchronously triggering actions.\n\n- **Terminating:** Ensures that the actor's state will be saved and performs all necessary cleanups.","ref":"Actors.Actor.Entity.html#module-behavior","title":"Behavior - Actors.Actor.Entity","type":"module"},{"doc":"The following client APIs are available for interaction:\n\n- `start_link/1`: Starts the entity for a given actor state.\n- `get_state/2`: Retrieves the actor state directly from memory.\n- `invoke/3`: Synchronously invokes an action on an actor.\n- `invoke_async/3`: Asynchronously invokes an action on an actor.\n\n#","ref":"Actors.Actor.Entity.html#module-client-apis","title":"Client APIs - Actors.Actor.Entity","type":"module"},{"doc":"- `init/1`: Initializes the actor entity.\n- `handle_continue/2`: Handles asynchronous events during the actor lifecycle.\n- `handle_call/3`: Handles synchronous calls to the actor.\n- `handle_cast/2`: Handles asynchronous casts to the actor.\n- `handle_info/2`: Handles informational messages.\n- `terminate/2`: Terminates the actor entity.","ref":"Actors.Actor.Entity.html#module-callbacks","title":"Callbacks - Actors.Actor.Entity","type":"module"},{"doc":"start_link/1: Starts the entity for a given actor state.\n\nget_state/2: Retrieves the actor state directly from memory.\n\ninvoke/3: Synchronously invokes an action on an actor.\n\ninvoke_async/3: Asynchronously invokes an action on an actor.","ref":"Actors.Actor.Entity.html#module-client-apis","title":"Client APIs - Actors.Actor.Entity","type":"module"},{"doc":"To use this module, start the actor by calling `start_link/1` with an initial actor state.\nInteraction with the actor is facilitated through the provided client APIs such as `get_state/2`, `invoke/3`, and `invoke_async/3`.","ref":"Actors.Actor.Entity.html#module-usage","title":"Usage - Actors.Actor.Entity","type":"module"},{"doc":"```elixir\n{:ok, actor} = Actors.Actor.Entity.start_link(%EntityState{actor: %Actor{id: %ActorId{name: \"example\"}}})\nstate = Actors.Actor.Entity.get_state(actor)\n{:ok, result} = Actors.Actor.Entity.invoke(actor, %InvocationRequest{action: :some_action})\n```\n\nNote: Ensure proper configuration and integration with the distributed system for seamless actor interactions.","ref":"Actors.Actor.Entity.html#module-example","title":"Example - Actors.Actor.Entity","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Actors.Actor.Entity.html#child_spec/1","title":"Actors.Actor.Entity.child_spec/1","type":"function"},{"doc":"Retrieve the Actor state direct from memory.","ref":"Actors.Actor.Entity.html#get_state/2","title":"Actors.Actor.Entity.get_state/2","type":"function"},{"doc":"Synchronously invokes an Action on an Actor.","ref":"Actors.Actor.Entity.html#invoke/3","title":"Actors.Actor.Entity.invoke/3","type":"function"},{"doc":"Asynchronously invokes an Action on an Actor.","ref":"Actors.Actor.Entity.html#invoke_async/3","title":"Actors.Actor.Entity.invoke_async/3","type":"function"},{"doc":"Starts the entity for a given actor state.","ref":"Actors.Actor.Entity.html#start_link/1","title":"Actors.Actor.Entity.start_link/1","type":"function"},{"doc":"`EntityState` Represents the internal state of an Actor.","ref":"Actors.Actor.Entity.EntityState.html","title":"Actors.Actor.Entity.EntityState","type":"module"},{"doc":"","ref":"Actors.Actor.Entity.EntityState.html#pack/1","title":"Actors.Actor.Entity.EntityState.pack/1","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.EntityState.html#unpack/1","title":"Actors.Actor.Entity.EntityState.unpack/1","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.EntityState.html#t:t/0","title":"Actors.Actor.Entity.EntityState.t/0","type":"type"},{"doc":"Handles Invocation functions for Actor Entity\nAll the public functions here assumes they are executing inside a GenServer","ref":"Actors.Actor.Entity.Invocation.html","title":"Actors.Actor.Entity.Invocation","type":"module"},{"doc":"","ref":"Actors.Actor.Entity.Invocation.html#do_broadcast/3","title":"Actors.Actor.Entity.Invocation.do_broadcast/3","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.Invocation.html#do_side_effects/2","title":"Actors.Actor.Entity.Invocation.do_side_effects/2","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.Invocation.html#handle_timers/3","title":"Actors.Actor.Entity.Invocation.handle_timers/3","type":"function"},{"doc":"Handles the invocation of actions on an Actor Entity.","ref":"Actors.Actor.Entity.Invocation.html#invoke/2","title":"Actors.Actor.Entity.Invocation.invoke/2","type":"function"},{"doc":"- `invocation` (%InvocationRequest{}): A struct representing the invocation request, including details about the actor, action, and payload.\n- `opts` (Keyword.t): Additional options for the invocation.","ref":"Actors.Actor.Entity.Invocation.html#invoke/2-parameters","title":"Parameters - Actors.Actor.Entity.Invocation.invoke/2","type":"function"},{"doc":"- `{:reply, result, new_state}`: If the invocation is successful, returns a tuple containing the reply result, and the updated entity state.\n- `{:noreply, new_state}`: If the invocation is successful, but there is no specific reply result.\n- `{:noreply, new_state, opts}`: If the invocation is successful and includes additional options.\n- `{:error, reason, new_state}`: If there is an error during the invocation, returns a tuple with the reason for the error and the updated entity state.","ref":"Actors.Actor.Entity.Invocation.html#invoke/2-returns","title":"Returns - Actors.Actor.Entity.Invocation.invoke/2","type":"function"},{"doc":"The `invoke/2` function is responsible for handling the invocation of actions on an Actor Entity.\nIt verifies authorization, finds the appropriate action to execute, and delegates the invocation to the corresponding interface.\n\nThe function performs the following steps:\n\n1. **Authorization Check:** Checks if the invocation is authorized based on the actor's actions and timers.\n\n2. **Span Context Handling:** Uses OpenTelemetry for tracing and creates a new span for the invocation.\n\n3. **Find Request by Action:** Determines the appropriate interface and builds the request based on the action.\n\n4. **Invocation Host Handling:** Delegates the invocation to the selected interface's `invoke_host/3` function.\n\n5. **Handle Response:** Processes the response from the invocation, handles side effects, and updates the entity state.\n\n6. **Checkpoint:** Optionally performs a checkpoint operation to record the state revision.","ref":"Actors.Actor.Entity.Invocation.html#invoke/2-behavior","title":"Behavior - Actors.Actor.Entity.Invocation.invoke/2","type":"function"},{"doc":"```elixir\ninvocation = %InvocationRequest{\n  actor: %Actor{id: %ActorId{name: \"example_actor\"}},\n  action_name: \"perform_action\",\n  payload: {:data, %{}},\n  caller: %ActorId{name: \"caller_actor\"}\n}\n\nopts = [span_ctx: OpenTelemetry.Ctx.new()]\n\ncase Actors.Actor.Entity.Invocation.invoke({invocation, opts}, entity_state) do\n  {:reply, result, new_state} ->\n    IO.puts(\"Invocation successful! Result: {result}\")\n  {:noreply, new_state} ->\n    IO.puts(\"Invocation successful! No specific reply.\")\n  {:error, reason, new_state} ->\n    IO.puts(\"Invocation failed. Reason: {reason}\")\nend\n```","ref":"Actors.Actor.Entity.Invocation.html#invoke/2-example","title":"Example - Actors.Actor.Entity.Invocation.invoke/2","type":"function"},{"doc":"Handles the initialization invocation for an Actor Entity.","ref":"Actors.Actor.Entity.Invocation.html#invoke_init/1","title":"Actors.Actor.Entity.Invocation.invoke_init/1","type":"function"},{"doc":"- `state` (%EntityState{}): The current state of the Actor Entity.","ref":"Actors.Actor.Entity.Invocation.html#invoke_init/1-parameters","title":"Parameters - Actors.Actor.Entity.Invocation.invoke_init/1","type":"function"},{"doc":"- `{:noreply, new_state}`: If the initialization is successful.\n- `{:noreply, new_state}`: If the actor has not registered any actions, indicating a warning.\n- `{:error, reason, new_state}`: If there is an error during the initialization, returns a tuple with the reason for the error and the updated entity state.","ref":"Actors.Actor.Entity.Invocation.html#invoke_init/1-returns","title":"Returns - Actors.Actor.Entity.Invocation.invoke_init/1","type":"function"},{"doc":"The `invoke_init/1` function is responsible for handling the initialization invocation of an Actor Entity. It checks if the actor has registered any actions and performs the following steps:\n\n1. **Action Registration Check:** Checks if the actor has registered any actions. If not, it logs a warning and returns `{:noreply, new_state}`.\n\n2. **Init Action Selection:** Filters the registered actions to find the initialization action (`init` or similar) and selects the first matching action.\n\n3. **Interface Invocation:** Invokes the selected initialization action using the appropriate interface.\n\n4. **Handling Initialization Result:** Processes the result of the initialization, updating the entity state accordingly.","ref":"Actors.Actor.Entity.Invocation.html#invoke_init/1-behavior","title":"Behavior - Actors.Actor.Entity.Invocation.invoke_init/1","type":"function"},{"doc":"```elixir\nstate = %EntityState{\n  system: %ActorSystem{name: \"example_system\"},\n  actor: %Actor{\n    id: %ActorId{name: \"example_actor\"},\n    actions: [\"init\", \"perform_action\"],\n    state: %{}\n  },\n  opts: %{}\n}\n\ncase Actors.Actor.Entity.Invocation.invoke_init(state) do\n  {:noreply, new_state} ->\n    IO.puts(\"Initialization successful!\")\n  {:error, reason, new_state} ->\n    IO.puts(\"Initialization failed. Reason: {reason}\")\nend\n```","ref":"Actors.Actor.Entity.Invocation.html#invoke_init/1-example","title":"Example - Actors.Actor.Entity.Invocation.invoke_init/1","type":"function"},{"doc":"Handles lifecycle functions for Actor Entity\nAll the public functions here assumes they are executing inside a GenServer","ref":"Actors.Actor.Entity.Lifecycle.html","title":"Actors.Actor.Entity.Lifecycle","type":"module"},{"doc":"","ref":"Actors.Actor.Entity.Lifecycle.html#checkpoint/2","title":"Actors.Actor.Entity.Lifecycle.checkpoint/2","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.Lifecycle.html#deactivate/1","title":"Actors.Actor.Entity.Lifecycle.deactivate/1","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.Lifecycle.html#get_state/2","title":"Actors.Actor.Entity.Lifecycle.get_state/2","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.Lifecycle.html#init/1","title":"Actors.Actor.Entity.Lifecycle.init/1","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.Lifecycle.html#load_state/1","title":"Actors.Actor.Entity.Lifecycle.load_state/1","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.Lifecycle.html#snapshot/1","title":"Actors.Actor.Entity.Lifecycle.snapshot/1","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.Lifecycle.html#terminate/2","title":"Actors.Actor.Entity.Lifecycle.terminate/2","type":"function"},{"doc":"`Entity.Supervisor` is the Supervisor of all Host Actors in the system.\nThis in turn is Partitioned using a PartitionSupervisor.\nWe use a hash function based on each Actor's child_spec to partition the actors\nso that the entire Supervisor is not overloaded and\nlessening the possibility of cascading failures.","ref":"Actors.Actor.Entity.Supervisor.html","title":"Actors.Actor.Entity.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Actors.Actor.Entity.Supervisor.html#child_spec/1","title":"Actors.Actor.Entity.Supervisor.child_spec/1","type":"function"},{"doc":"Adds a Actor to the dynamic supervisor.","ref":"Actors.Actor.Entity.Supervisor.html#lookup_or_create_actor/3","title":"Actors.Actor.Entity.Supervisor.lookup_or_create_actor/3","type":"function"},{"doc":"","ref":"Actors.Actor.Entity.Supervisor.html#start_link/1","title":"Actors.Actor.Entity.Supervisor.start_link/1","type":"function"},{"doc":"`Invoker` is responsible for making calls to the Host Function","ref":"Actors.Actor.Interface.html","title":"Actors.Actor.Interface","type":"behaviour"},{"doc":"","ref":"Actors.Actor.Interface.html#c:invoke_host/3","title":"Actors.Actor.Interface.invoke_host/3","type":"callback"},{"doc":"","ref":"Actors.Actor.Interface.html#t:default_methods/0","title":"Actors.Actor.Interface.default_methods/0","type":"type"},{"doc":"","ref":"Actors.Actor.Interface.html#t:payload/0","title":"Actors.Actor.Interface.payload/0","type":"type"},{"doc":"","ref":"Actors.Actor.Interface.html#t:state/0","title":"Actors.Actor.Interface.state/0","type":"type"},{"doc":"`Http` is responsible for the communication between the Proxy and the Host application\nwhen using the HTTP protocol.","ref":"Actors.Actor.Interface.Http.html","title":"Actors.Actor.Interface.Http","type":"module"},{"doc":"`InvocationScheduler` is a global process for the cluster that controls\nall Actions of type Schedule.\nThis process is global to allow that even after restarts of a process or restart\nof an application we will still be able to perform invocations to actors,\nwithout the need for persistent storage such as a database.","ref":"Actors.Actor.InvocationScheduler.html","title":"Actors.Actor.InvocationScheduler","type":"module"},{"doc":"","ref":"Actors.Actor.InvocationScheduler.html#child_spec/0","title":"Actors.Actor.InvocationScheduler.child_spec/0","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Actors.Actor.InvocationScheduler.html#child_spec/1","title":"Actors.Actor.InvocationScheduler.child_spec/1","type":"function"},{"doc":"","ref":"Actors.Actor.InvocationScheduler.html#next_hibernate_delay/0","title":"Actors.Actor.InvocationScheduler.next_hibernate_delay/0","type":"function"},{"doc":"","ref":"Actors.Actor.InvocationScheduler.html#schedule_fixed_invocations/1","title":"Actors.Actor.InvocationScheduler.schedule_fixed_invocations/1","type":"function"},{"doc":"","ref":"Actors.Actor.InvocationScheduler.html#schedule_invoke/2","title":"Actors.Actor.InvocationScheduler.schedule_invoke/2","type":"function"},{"doc":"","ref":"Actors.Actor.InvocationScheduler.html#start_link/0","title":"Actors.Actor.InvocationScheduler.start_link/0","type":"function"},{"doc":"This module provides functions for creating actor host pools for pooled actors.","ref":"Actors.Actor.Pool.html","title":"Actors.Actor.Pool","type":"module"},{"doc":"Creates an actor host pool for a given pooled actor.","ref":"Actors.Actor.Pool.html#create_actor_host_pool/2","title":"Actors.Actor.Pool.create_actor_host_pool/2","type":"function"},{"doc":"- `actor`: The actor for which the host pool should be created.\n- `opts`: Additional options for the host pool.","ref":"Actors.Actor.Pool.html#create_actor_host_pool/2-parameters","title":"Parameters - Actors.Actor.Pool.create_actor_host_pool/2","type":"function"},{"doc":"Returns a list of `HostActor` structs representing the hosts in the pool.","ref":"Actors.Actor.Pool.html#create_actor_host_pool/2-returns","title":"Returns - Actors.Actor.Pool.create_actor_host_pool/2","type":"function"},{"doc":"","ref":"Actors.Actor.Pubsub.html","title":"Actors.Actor.Pubsub","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Actors.Actor.Pubsub.html#child_spec/1","title":"Actors.Actor.Pubsub.child_spec/1","type":"function"},{"doc":"","ref":"Actors.Actor.Pubsub.html#publish/3","title":"Actors.Actor.Pubsub.publish/3","type":"function"},{"doc":"","ref":"Actors.Actor.Pubsub.html#start_link/1","title":"Actors.Actor.Pubsub.start_link/1","type":"function"},{"doc":"Subscribes a specific actor to a topic","ref":"Actors.Actor.Pubsub.html#subscribe/4","title":"Actors.Actor.Pubsub.subscribe/4","type":"function"},{"doc":"Custom PubsubDispatcher dispatcher to send subscriber metadata with the message.","ref":"Actors.Actor.PubsubDispatcher.html","title":"Actors.Actor.PubsubDispatcher","type":"module"},{"doc":"`StateManager` Implements behavior that allows an Actor's state to be saved\nto persistent storage using database drivers.","ref":"Actors.Actor.StateManager.html","title":"Actors.Actor.StateManager","type":"module"},{"doc":"","ref":"Actors.Actor.StateManager.html#is_new?/2","title":"Actors.Actor.StateManager.is_new?/2","type":"function"},{"doc":"","ref":"Actors.Actor.StateManager.html#load/1","title":"Actors.Actor.StateManager.load/1","type":"function"},{"doc":"","ref":"Actors.Actor.StateManager.html#load/2","title":"Actors.Actor.StateManager.load/2","type":"function"},{"doc":"","ref":"Actors.Actor.StateManager.html#load_all/1","title":"Actors.Actor.StateManager.load_all/1","type":"function"},{"doc":"","ref":"Actors.Actor.StateManager.html#load_by_interval/3","title":"Actors.Actor.StateManager.load_by_interval/3","type":"function"},{"doc":"","ref":"Actors.Actor.StateManager.html#save/3","title":"Actors.Actor.StateManager.save/3","type":"function"},{"doc":"","ref":"Actors.Actor.StateManager.html#save_async/3","title":"Actors.Actor.StateManager.save_async/3","type":"function"},{"doc":"`StateManager.Behaviour` Defines behavior that allows an Actor's state\nto be saved to persistent storage using database drivers.","ref":"Actors.Actor.StateManager.Behaviour.html","title":"Actors.Actor.StateManager.Behaviour","type":"behaviour"},{"doc":"","ref":"Actors.Actor.StateManager.Behaviour.html#c:is_new?/2","title":"Actors.Actor.StateManager.Behaviour.is_new?/2","type":"callback"},{"doc":"","ref":"Actors.Actor.StateManager.Behaviour.html#c:load/1","title":"Actors.Actor.StateManager.Behaviour.load/1","type":"callback"},{"doc":"","ref":"Actors.Actor.StateManager.Behaviour.html#c:load/2","title":"Actors.Actor.StateManager.Behaviour.load/2","type":"callback"},{"doc":"","ref":"Actors.Actor.StateManager.Behaviour.html#c:load_all/1","title":"Actors.Actor.StateManager.Behaviour.load_all/1","type":"callback"},{"doc":"","ref":"Actors.Actor.StateManager.Behaviour.html#c:load_by_interval/3","title":"Actors.Actor.StateManager.Behaviour.load_by_interval/3","type":"callback"},{"doc":"","ref":"Actors.Actor.StateManager.Behaviour.html#c:save/3","title":"Actors.Actor.StateManager.Behaviour.save/3","type":"callback"},{"doc":"","ref":"Actors.Actor.StateManager.Behaviour.html#c:save_async/3","title":"Actors.Actor.StateManager.Behaviour.save_async/3","type":"callback"},{"doc":"`Config` defines methods that allow recovery of system settings","ref":"Actors.Config.html","title":"Actors.Config","type":"behaviour"},{"doc":"","ref":"Actors.Config.html#c:get/1","title":"Actors.Config.get/1","type":"callback"},{"doc":"","ref":"Actors.Config.html#c:load/0","title":"Actors.Config.load/0","type":"callback"},{"doc":"`Config.PersistentTermConfig` Implements the `Config` behavior\nto allow the retrieval of system variables\nthat will be included in the system configuration.","ref":"Actors.Config.PersistentTermConfig.html","title":"Actors.Config.PersistentTermConfig","type":"module"},{"doc":"Initializes the %K8s.Conn{} struct depending on the mix environment.","ref":"Actors.K8s.K8sConn.html","title":"Actors.K8s.K8sConn","type":"module"},{"doc":"","ref":"Actors.K8s.K8sConn.html#get/1","title":"Actors.K8s.K8sConn.get/1","type":"function"},{"doc":"","ref":"Actors.K8s.K8sConn.html#get!/1","title":"Actors.K8s.K8sConn.get!/1","type":"function"},{"doc":"`Node.Client` Uses the HTTP interface to communicate with the application\nthat owns the ActorHosts.","ref":"Actors.Node.Client.html","title":"Actors.Node.Client","type":"module"},{"doc":"Perform a DELETE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    delete(\"/users\")\n    delete(\"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\")\n    delete(client, \"/users\", query: [scope: \"admin\"])\n    delete(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#delete/3","title":"Actors.Node.Client.delete/3","type":"function"},{"doc":"Perform a DELETE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    delete!(\"/users\")\n    delete!(\"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\")\n    delete!(client, \"/users\", query: [scope: \"admin\"])\n    delete!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#delete!/3","title":"Actors.Node.Client.delete!/3","type":"function"},{"doc":"Perform a GET request.\n\nSee `request/1` or `request/2` for options definition.\n\n    get(\"/users\")\n    get(\"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\")\n    get(client, \"/users\", query: [scope: \"admin\"])\n    get(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#get/3","title":"Actors.Node.Client.get/3","type":"function"},{"doc":"Perform a GET request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    get!(\"/users\")\n    get!(\"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\")\n    get!(client, \"/users\", query: [scope: \"admin\"])\n    get!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#get!/3","title":"Actors.Node.Client.get!/3","type":"function"},{"doc":"Perform a HEAD request.\n\nSee `request/1` or `request/2` for options definition.\n\n    head(\"/users\")\n    head(\"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\")\n    head(client, \"/users\", query: [scope: \"admin\"])\n    head(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#head/3","title":"Actors.Node.Client.head/3","type":"function"},{"doc":"Perform a HEAD request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    head!(\"/users\")\n    head!(\"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\")\n    head!(client, \"/users\", query: [scope: \"admin\"])\n    head!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#head!/3","title":"Actors.Node.Client.head!/3","type":"function"},{"doc":"","ref":"Actors.Node.Client.html#invoke_host_actor/2","title":"Actors.Node.Client.invoke_host_actor/2","type":"function"},{"doc":"Perform a OPTIONS request.\n\nSee `request/1` or `request/2` for options definition.\n\n    options(\"/users\")\n    options(\"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\")\n    options(client, \"/users\", query: [scope: \"admin\"])\n    options(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#options/3","title":"Actors.Node.Client.options/3","type":"function"},{"doc":"Perform a OPTIONS request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    options!(\"/users\")\n    options!(\"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\")\n    options!(client, \"/users\", query: [scope: \"admin\"])\n    options!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#options!/3","title":"Actors.Node.Client.options!/3","type":"function"},{"doc":"Perform a PATCH request.\n\nSee `request/1` or `request/2` for options definition.\n\n    patch(\"/users\", %{name: \"Jon\"})\n    patch(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch(client, \"/users\", %{name: \"Jon\"})\n    patch(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Actors.Node.Client.html#patch/4","title":"Actors.Node.Client.patch/4","type":"function"},{"doc":"Perform a PATCH request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    patch!(\"/users\", %{name: \"Jon\"})\n    patch!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    patch!(client, \"/users\", %{name: \"Jon\"})\n    patch!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Actors.Node.Client.html#patch!/4","title":"Actors.Node.Client.patch!/4","type":"function"},{"doc":"Perform a POST request.\n\nSee `request/1` or `request/2` for options definition.\n\n    post(\"/users\", %{name: \"Jon\"})\n    post(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post(client, \"/users\", %{name: \"Jon\"})\n    post(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Actors.Node.Client.html#post/4","title":"Actors.Node.Client.post/4","type":"function"},{"doc":"Perform a POST request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    post!(\"/users\", %{name: \"Jon\"})\n    post!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    post!(client, \"/users\", %{name: \"Jon\"})\n    post!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Actors.Node.Client.html#post!/4","title":"Actors.Node.Client.post!/4","type":"function"},{"doc":"Perform a PUT request.\n\nSee `request/1` or `request/2` for options definition.\n\n    put(\"/users\", %{name: \"Jon\"})\n    put(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put(client, \"/users\", %{name: \"Jon\"})\n    put(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Actors.Node.Client.html#put/4","title":"Actors.Node.Client.put/4","type":"function"},{"doc":"Perform a PUT request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    put!(\"/users\", %{name: \"Jon\"})\n    put!(\"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])\n    put!(client, \"/users\", %{name: \"Jon\"})\n    put!(client, \"/users\", %{name: \"Jon\"}, query: [scope: \"admin\"])","ref":"Actors.Node.Client.html#put!/4","title":"Actors.Node.Client.put!/4","type":"function"},{"doc":"Perform a request.","ref":"Actors.Node.Client.html#request/2","title":"Actors.Node.Client.request/2","type":"function"},{"doc":"- `:method` - the request method, one of [`:head`, `:get`, `:delete`, `:trace`, `:options`, `:post`, `:put`, `:patch`]\n- `:url` - either full url e.g. \"http://example.com/some/path\" or just \"/some/path\" if using `Tesla.Middleware.BaseUrl`\n- `:query` - a keyword list of query params, e.g. `[page: 1, per_page: 100]`\n- `:headers` - a keyworld list of headers, e.g. `[{\"content-type\", \"text/plain\"}]`\n- `:body` - depends on used middleware:\n    - by default it can be a binary\n    - if using e.g. JSON encoding middleware it can be a nested map\n    - if adapter supports it it can be a Stream with any of the above\n- `:opts` - custom, per-request middleware or adapter options","ref":"Actors.Node.Client.html#request/2-options","title":"Options - Actors.Node.Client.request/2","type":"function"},{"doc":"ExampleApi.request(method: :get, url: \"/users/path\")\n\n    # use shortcut methods\n    ExampleApi.get(\"/users/1\")\n    ExampleApi.post(client, \"/users\", %{name: \"Jon\"})","ref":"Actors.Node.Client.html#request/2-examples","title":"Examples - Actors.Node.Client.request/2","type":"function"},{"doc":"Perform request and raise in case of error.\n\nThis is similar to `request/2` behaviour from Tesla 0.x\n\nSee `request/2` for list of available options.","ref":"Actors.Node.Client.html#request!/2","title":"Actors.Node.Client.request!/2","type":"function"},{"doc":"Perform a TRACE request.\n\nSee `request/1` or `request/2` for options definition.\n\n    trace(\"/users\")\n    trace(\"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\")\n    trace(client, \"/users\", query: [scope: \"admin\"])\n    trace(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#trace/3","title":"Actors.Node.Client.trace/3","type":"function"},{"doc":"Perform a TRACE request.\n\nSee `request!/1` or `request!/2` for options definition.\n\n    trace!(\"/users\")\n    trace!(\"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\")\n    trace!(client, \"/users\", query: [scope: \"admin\"])\n    trace!(client, \"/users\", body: %{name: \"Jon\"})","ref":"Actors.Node.Client.html#trace!/3","title":"Actors.Node.Client.trace!/3","type":"function"},{"doc":"Options that may be passed to a request function. See `request/2` for detailed descriptions.","ref":"Actors.Node.Client.html#t:option/0","title":"Actors.Node.Client.option/0","type":"type"},{"doc":"The `ActorRegistry` module provides a registry for actor entities.\n\nIt allows for registering and looking up actors and also provides\nmethods for adding and removing invocation requests for actors.","ref":"Actors.Registry.ActorRegistry.html","title":"Actors.Registry.ActorRegistry","type":"module"},{"doc":"Retrieves a list of hosts associated with a given actor.\nParameters\n\nactor_id (%ActorId{}): The ID of the actor.\nopts (Keyword, optional): Options for the function.\n    :parent (boolean): If true, filter hosts by parent; otherwise, filter by name.\n\nReturns\n\n{:ok, hosts}: A tuple with :ok and a list of hosts.\n{:not_found, []}: If no hosts are found.\n\nExamples\n\n```elixir\nactor_id = %ActorId{parent: \"parent\", name: \"name\"}\nopts = [parent: true]\n{:ok, hosts} = Actors.Registry.ActorRegistry.get_hosts_by_actor(actor_id, opts)\n\n```","ref":"Actors.Registry.ActorRegistry.html#get_hosts_by_actor/2","title":"Actors.Registry.ActorRegistry.get_hosts_by_actor/2","type":"function"},{"doc":"Fetch current entities of the service.\nReturns `HostActor` with Host and specific actor.","ref":"Actors.Registry.ActorRegistry.html#lookup/2","title":"Actors.Registry.ActorRegistry.lookup/2","type":"function"},{"doc":"Register `member` entities to the ActorRegistry.\nReturns `Cluster` with all Host members.","ref":"Actors.Registry.ActorRegistry.html#register/1","title":"Actors.Registry.ActorRegistry.register/1","type":"function"},{"doc":"iex> hosts = [%HostActor{node: Node.self(), actor: actor, opts: []}}]\n    iex> ActorRegistry.register(hosts)\n    :ok","ref":"Actors.Registry.ActorRegistry.html#register/1-examples","title":"Examples - Actors.Registry.ActorRegistry.register/1","type":"function"},{"doc":"`HostActor` Defines the type of Actor that will be registered in `ActorRegistry`.","ref":"Actors.Registry.HostActor.html","title":"Actors.Registry.HostActor","type":"module"},{"doc":"","ref":"Actors.Registry.HostActor.html#t:t/0","title":"Actors.Registry.HostActor.t/0","type":"type"},{"doc":"`Random` implements the `Actors.Registry.LoadBalance.Strategy` behavior\nby searching the actors randomly, that is,\nif you have registered the same actor in N nodes and try to invoke it,\nthe Registry can, if configured,\nuse the random strategy to locate one of them in any of the nodes in a random way,\nas the name suggests.","ref":"Actors.Registry.LoadBalance.Strategies.Random.html","title":"Actors.Registry.LoadBalance.Strategies.Random","type":"module"},{"doc":"`RoundRobin` implements the `Actors.Registry.LoadBalance.Strategy` behavior\nby searching the actors using a RoundRobin strategy,\nthat is, it will try to distribute the load equally among the nodes.","ref":"Actors.Registry.LoadBalance.Strategies.RoundRobin.html","title":"Actors.Registry.LoadBalance.Strategies.RoundRobin","type":"module"},{"doc":"`LoadBalance.Strategy` Define an interface to allow the search of actors\nin the Distributed Registry.","ref":"Actors.Registry.LoadBalance.Strategy.html","title":"Actors.Registry.LoadBalance.Strategy","type":"behaviour"},{"doc":"","ref":"Actors.Registry.LoadBalance.Strategy.html#c:next_host/2","title":"Actors.Registry.LoadBalance.Strategy.next_host/2","type":"callback"},{"doc":"","ref":"Actors.Registry.LoadBalance.Strategy.html#t:hosts/0","title":"Actors.Registry.LoadBalance.Strategy.hosts/0","type":"type"},{"doc":"","ref":"Actors.Registry.LoadBalance.Strategy.html#t:opts/0","title":"Actors.Registry.LoadBalance.Strategy.opts/0","type":"type"},{"doc":"`LoadBalance` to call hosts","ref":"Actors.Registry.LoadBalancer.html","title":"Actors.Registry.LoadBalancer","type":"module"},{"doc":"","ref":"Actors.Registry.LoadBalancer.html#next_host/1","title":"Actors.Registry.LoadBalancer.next_host/1","type":"function"},{"doc":"`Acl` is a acces control list helper module","ref":"Actors.Security.Acl.html","title":"Actors.Security.Acl","type":"behaviour"},{"doc":"","ref":"Actors.Security.Acl.html#c:get_policies!/0","title":"Actors.Security.Acl.get_policies!/0","type":"callback"},{"doc":"","ref":"Actors.Security.Acl.html#c:is_authorized?/2","title":"Actors.Security.Acl.is_authorized?/2","type":"callback"},{"doc":"","ref":"Actors.Security.Acl.html#c:load_acl_policies/1","title":"Actors.Security.Acl.load_acl_policies/1","type":"callback"},{"doc":"","ref":"Actors.Security.Acl.html#t:base_policies_path/0","title":"Actors.Security.Acl.base_policies_path/0","type":"type"},{"doc":"","ref":"Actors.Security.Acl.html#t:invocation/0","title":"Actors.Security.Acl.invocation/0","type":"type"},{"doc":"","ref":"Actors.Security.Acl.html#t:policies/0","title":"Actors.Security.Acl.policies/0","type":"type"},{"doc":"","ref":"Actors.Security.Acl.DefaultAclManager.html","title":"Actors.Security.Acl.DefaultAclManager","type":"module"},{"doc":"`DefaultAclManager`","ref":"Actors.Security.Acl.DefaultAclManager.html#load_acl_policies/1","title":"Actors.Security.Acl.DefaultAclManager.load_acl_policies/1","type":"function"},{"doc":"","ref":"Actors.Security.Acl.Policy.html","title":"Actors.Security.Acl.Policy","type":"module"},{"doc":"","ref":"Actors.Security.Acl.Policy.html#t:t/0","title":"Actors.Security.Acl.Policy.t/0","type":"type"},{"doc":"`AclEvaluator`is responsible for evaluating whether a request can be accepted\nor not based on the informed Access Control List policy.","ref":"Actors.Security.Acl.Rules.AclEvaluator.html","title":"Actors.Security.Acl.Rules.AclEvaluator","type":"module"},{"doc":"Applies a policy on a request by evaluating whether the request should pass or not.","ref":"Actors.Security.Acl.Rules.AclEvaluator.html#eval/3","title":"Actors.Security.Acl.Rules.AclEvaluator.eval/3","type":"function"},{"doc":"iex> Actors.Security.Acl.Rules.AclEvaluator.eval(\n    ...> %Actors.Security.Acl.Policy{\n    ...>   name: \"the-police\",\n    ...>   type: :allow,\n    ...>   actors: [\"*\"],\n    ...>   actions: [\"get\"],\n    ...>   actor_systems: [\"*\"]\n    ...> },\n    ...> %Eigr.Functions.Protocol.InvocationRequest{\n    ...>    actor: %Eigr.Functions.Protocol.Actors.Actor{id: %Eigr.Functions.Protocol.Actors.ActorId{name: \"joe\"}},\n    ...>    action_name: \"get\",\n    ...>    caller: %Eigr.Functions.Protocol.Actors.ActorId{name: \"robert\", system: \"actor-system\"}\n    ...> }\n    ...> )\n    true","ref":"Actors.Security.Acl.Rules.AclEvaluator.html#eval/3-example","title":"Example - Actors.Security.Acl.Rules.AclEvaluator.eval/3","type":"function"},{"doc":"This module must be used by initializing the container via initContainers\n\n```\ninitContainers:\n  - args:\n      - eval\n      - Kompost.Webhooks.bootstrap_tls(:prod, \"tls-certs\")\n    image: docker.io/eigr/spawn-proxy:1.1.0\n    name: init-certificates\nserviceAccountName: kompost\nvolumes:\n  - name: certs\n    secret:\n      optional: true\n      secretName: tls-certs\n```","ref":"Actors.Security.Tls.Initializer.html","title":"Actors.Security.Tls.Initializer","type":"module"},{"doc":"","ref":"Actors.Security.Tls.Initializer.html#bootstrap_tls/5","title":"Actors.Security.Tls.Initializer.bootstrap_tls/5","type":"function"},{"doc":"","ref":"Actors.SplitBrainDetector.html","title":"Actors.SplitBrainDetector","type":"behaviour"},{"doc":"","ref":"Actors.SplitBrainDetector.html#c:check_network_partition/3","title":"Actors.SplitBrainDetector.check_network_partition/3","type":"callback"},{"doc":"","ref":"Actors.SplitBrainDetector.html#c:check_network_partition!/3","title":"Actors.SplitBrainDetector.check_network_partition!/3","type":"callback"},{"doc":"","ref":"Actors.SplitBrainDetector.html#t:actor_id/0","title":"Actors.SplitBrainDetector.actor_id/0","type":"type"},{"doc":"","ref":"Actors.SplitBrainDetector.html#t:node_id/0","title":"Actors.SplitBrainDetector.node_id/0","type":"type"},{"doc":"","ref":"Actors.SplitBrainDetector.html#t:status/0","title":"Actors.SplitBrainDetector.status/0","type":"type"},{"doc":"Documentation for `Sidecar`.","ref":"Sidecar.html","title":"Sidecar","type":"module"},{"doc":"This module was copied and adapted from this project https://github.com/straw-hat-team/beam-monorepo\n\nCatches `SIGTERM` signal to gracefully stop the system.\nBy default, a `SIGTERM` signal triggers a `System.stop/0`. When running in a Kubernetes-managed cluster or an\ninfrastructure alike, the nodes will be joining and leaving the cluster when auto-scaling is enabled or when deploying\nnew version of the application.\nYou should let the system to gracefully stop in combination with readiness probe you can make sure that the system is\nready to stop without a relying on a `SIGKILL` signal.\n\nYou can read more about this at [Termination of Pods](https://kubernetes.io/docs/concepts/workloads/pods/pod-lifecycle/#pod-termination).\nWhen a `SIGTERM` is received, the handler enters \"connection draining mode\", during which `get_status/0` starts\nreturning `:draining` status, and then, after a delay, the handler calls `System.stop/1`, carefully halting the Erlang VM\nand starts returning `:stopping` status.","ref":"Sidecar.GracefulShutdown.html","title":"Sidecar.GracefulShutdown","type":"module"},{"doc":"Returns a specification to start `Sidecar.GracefulShutdown` under a supervisor.\nSee the \"Child specification\" section in the Supervisor module for more detailed information.","ref":"Sidecar.GracefulShutdown.html#child_spec/1","title":"Sidecar.GracefulShutdown.child_spec/1","type":"function"},{"doc":"Returns the status of the system.","ref":"Sidecar.GracefulShutdown.html#get_status/0","title":"Sidecar.GracefulShutdown.get_status/0","type":"function"},{"doc":"Returns true if the system is in the `:running` status.","ref":"Sidecar.GracefulShutdown.html#running?/0","title":"Sidecar.GracefulShutdown.running?/0","type":"function"},{"doc":"Configuration options for the server.\n- `shutdown_delay_ms`: milliseconds before draining the VM after `SIGTERM`. milliseconds between start of connection draining\n    and ordered shut-down using `System.stop/0`.\n- `notify_pid`: process ID to notify when the server started draining.\n- `init_stop?`: whether to call `:System.stop/1` when a `SIGTERM` is received. Useful for testing since we don't want\n    to stop the VM when testing. **Be careful when using this option since it should always be `true` in production.**","ref":"Sidecar.GracefulShutdown.html#t:opts/0","title":"Sidecar.GracefulShutdown.opts/0","type":"type"},{"doc":"The state of system.\n- `:draining` - the node is draining and will be shutdown after the delay.\n- `:stopping` - the node is shutting down.\n- `:running` - the node is accepting work.","ref":"Sidecar.GracefulShutdown.html#t:status/0","title":"Sidecar.GracefulShutdown.status/0","type":"type"},{"doc":"![Sepp](docs/images/sepp-elixir-254-400.png#gh-light-mode-only)\n![Sepp](docs/images/sepp-elixir-254-400.png#gh-dark-mode-only)\n\n**Actor Mesh Distributed Runtime**\n\n![ci](https://github.com/eigr/spawn/actions/workflows/ci.yaml/badge.svg)\n![last commit](https://img.shields.io/github/last-commit/eigr/spawn?style=social)\n[![join discord](https://badgen.net/badge/discord/Join%20Eigr%20on%20Discord/discord?icon=discord&label&color=blue)](https://discord.gg/2PcshvfS93)\n[![twitter](https://badgen.net/badge/twitter/@eigr_io/blue?label&icon=twitter)](https://twitter.com/eigr_io)\n\n### **[Website](https://eigr.io)** • **[Getting Started](docs/getting_started.md)** • **[SDKs](docs/sdks.md)** • **[Documentation](https://eigr.io/docs/projects-spawn/spawn-introduction/)** • **[Blog](https://eigr.io/blog/)**","ref":"Spawn.html","title":"Spawn","type":"module"},{"doc":"Since UC Berkeley published its [Cloud Programming Simplified: A Berkeley View on\nServerless Computing](https://www2.eecs.berkeley.edu/Pubs/TechRpts/2019/EECS-2019-3.pdf) in the year of 2019, several solutions for Stateful Serverless and Durable Computing are emerging on the market.\nOriginally coming from some contributors to the [Cloudstate](https://github.com/cloudstateio/cloudstate), Spawn stands as the Erlang world's answer to the challenges of durable serverless computing. \n\nUnlike serverless runtimes hidden in cloud providers' black boxes, Spawn is an open-source serverless runtime designed for both cloud and on-premises environments. Powered by [BEAM](https://www.erlang.org/blog/a-brief-beam-primer/) and [ERTS](https://www.erlang.org/doc/apps/erts/), Spawn offers a polyglot programming model for crafting versatile applications. It's **BEAM** for all.\n\nExplore the potential of Erlang, regardless of your preferred programming language, to swiftly accomplish your business objectives. Check out our [documentation](docs/index.md) to [get started](docs/getting_started.md).","ref":"Spawn.html#module-overview","title":"Overview - Spawn","type":"module"},{"doc":"> **_Tip:_** Serverless is not just FaaS\n\nServerless computing is a cloud computing execution model characterized by the automatic management of infrastructure, allowing developers to focus solely on code without the need to provision or manage servers explicitly. While commonly associated with cloud services, it's important to note that some serverless products operate independently of specific cloud providers, like Spawn. In these cases, the fundamental principles of automatic scaling, pay-per-execution pricing, and the abstraction of infrastructure complexities still apply, offering developers a serverless experience without exclusive reliance on a particular cloud platform.\n\nWhile serverless is often associated with Function as a Service (FaaS), where functions are the unit of deployment and execution, serverless computing can extend beyond just FaaS. In a broader sense, serverless includes services like backend-as-a-service (BaaS) and other cloud offerings where developers can build and deploy applications without dealing with the underlying infrastructure.\n\nThe key features of serverless computing include automatic scaling, pay-per-execution pricing, and a shift of operational responsibilities from the developer to the cloud provider or underlying runtime. Developers can focus more on writing code and implementing business logic, leaving the infrastructure management, scaling, and maintenance to the upper runtime.\n\nSpawn extends this Serverless computing model by considering that there is an infrastructure layer in all application code, such as state management, configuration management, integration flows with other applications, connection pooling, and so on, and that this code infrastructure must also be managed by the runtime, freeing the developer so that he can focus much more on his business objectives directly.\n\nThis also brings security to CEOs and CTOs as well as product managers that the central objectives of their companies will be achieved within a shorter period of time, thus increasing their competitiveness against their competitors. This without compromising technical quality and scalability to support your products running in production.","ref":"Spawn.html#module-demystifying-the-serverless-computing-model","title":"Demystifying the Serverless Computing Model - Spawn","type":"module"},{"doc":"* https://eigr.io/blog/spawn-the-actor-mesh/\n\n* https://eigr.io/blog/beyond-monoliths-and-microservices/\n\n* https://eigr.io/blog/distributed-elixir-made-easy-with-spawn/","ref":"Spawn.html#module-latest-blogs","title":"Latest Blogs - Spawn","type":"module"},{"doc":"Watch some insightful talks on Eigr Community and Spawn:\n\n- **Marcel Lanz on ACM SIGPLAN - Erlang 2021 - Lightning Talk**: https://www.youtube.com/watch?v=jVf0QqNb3qk _(English)_\n- **Marcel Lanz on Code Beam Europe 2022**: https://youtu.be/jgR7Oc_GXAg _(English)_\n- **Adriano Santos on Code Beam BR 2022**: https://www.youtube.com/watch?v=dXp0lyfmV_0&list=PLa5zLmv3pgCCiyWq8gltVAt2vTGuHz3eG&index=3 _(Portuguese)_\n- **Adriano Santos ElugSP 2023**: https://www.youtube.com/watch?v=MKTqiAtpK1E _(Portuguese)_\n- **Elias Arruda on Elixir CWB (Curitiba)**: https://www.youtube.com/live/yE_uWbPEWnI?si=5L3SORG3PERZpQ4V&t=463 _(Portuguese)_","ref":"Spawn.html#module-engaging-talks","title":"Engaging Talks - Spawn","type":"module"},{"doc":"Say hello to Sepp, our cyberpunk mascot and proud [Ibex](https://alpshiking.swisshikingvacations.com/spotlight-on-the-ibex/). Sepp, hailing from the Swiss Alps, loves wandering through the Eiger mountains. Despite occasional Viking-like bluntness, he's a helpful companion who traces his lineage back to Viking times.\n\nThe name \"Sepp\" is indeed a German diminutive or nickname for the name Joseph. It's similar to how \"Joe\" is used as a nickname for Joseph in English. In German-speaking regions, it's common for Joseph to be affectionately referred to as \"Sepp.\"\n\nThis is our little tribute to one of the creators of the Erlang programming language, [Joe Armstrong](https://en.wikipedia.org/wiki/Joe_Armstrong_(programmer)).\n\n![Sepp Rules](docs/images/sepp-rules-254-400.png#gh-light-mode-only)\n![Sepp Rules](docs/images/sepp-rules-254-400.png#gh-dark-mode-only)\n\nWrite less code and be happy! 🌐🚀","ref":"Spawn.html#module-meet-sepp-our-cyberpunk-mascot","title":"Meet Sepp: Our Cyberpunk Mascot - Spawn","type":"module"},{"doc":"","ref":"Spawn.Cache.LookupCache.html","title":"Spawn.Cache.LookupCache","type":"module"},{"doc":"A convenience function for getting the primary storage cache.","ref":"Spawn.Cache.LookupCache.html#__primary__/0","title":"Spawn.Cache.LookupCache.__primary__/0","type":"function"},{"doc":"A convenience function to get the node of the given `key`.","ref":"Spawn.Cache.LookupCache.html#get_node/1","title":"Spawn.Cache.LookupCache.get_node/1","type":"function"},{"doc":"A convenience function for joining the cache to the cluster.","ref":"Spawn.Cache.LookupCache.html#join_cluster/0","title":"Spawn.Cache.LookupCache.join_cluster/0","type":"function"},{"doc":"A convenience function for removing the cache from the cluster.","ref":"Spawn.Cache.LookupCache.html#leave_cluster/0","title":"Spawn.Cache.LookupCache.leave_cluster/0","type":"function"},{"doc":"A convenience function for getting the cluster nodes.","ref":"Spawn.Cache.LookupCache.html#nodes/0","title":"Spawn.Cache.LookupCache.nodes/0","type":"function"},{"doc":"This is the cache for the primary storage.","ref":"Spawn.Cache.LookupCache.Primary.html","title":"Spawn.Cache.LookupCache.Primary","type":"module"},{"doc":"A convenience function for retrieving the current generations.","ref":"Spawn.Cache.LookupCache.Primary.html#generations/0","title":"Spawn.Cache.LookupCache.Primary.generations/0","type":"function"},{"doc":"A convenience function for creating new generations.","ref":"Spawn.Cache.LookupCache.Primary.html#new_generation/1","title":"Spawn.Cache.LookupCache.Primary.new_generation/1","type":"function"},{"doc":"A convenience function for retrieving the newer generation.","ref":"Spawn.Cache.LookupCache.Primary.html#newer_generation/0","title":"Spawn.Cache.LookupCache.Primary.newer_generation/0","type":"function"},{"doc":"A convenience function for reset the GC timer.","ref":"Spawn.Cache.LookupCache.Primary.html#reset_generation_timer/0","title":"Spawn.Cache.LookupCache.Primary.reset_generation_timer/0","type":"function"},{"doc":"","ref":"Spawn.Cluster.Node.Client.html","title":"Spawn.Cluster.Node.Client","type":"module"},{"doc":"Defines a distributed registry for all process","ref":"Spawn.Cluster.Node.Registry.html","title":"Spawn.Cluster.Node.Registry","type":"module"},{"doc":"","ref":"Spawn.Cluster.Node.Registry.html#child_spec/0","title":"Spawn.Cluster.Node.Registry.child_spec/0","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\nSee `Supervisor`.","ref":"Spawn.Cluster.Node.Registry.html#child_spec/1","title":"Spawn.Cluster.Node.Registry.child_spec/1","type":"function"},{"doc":"Check if Process is alive.","ref":"Spawn.Cluster.Node.Registry.html#is_alive?/2","title":"Spawn.Cluster.Node.Registry.is_alive?/2","type":"function"},{"doc":"List all alive PIDs from given registry module.","ref":"Spawn.Cluster.Node.Registry.html#list_actor_pids/1","title":"Spawn.Cluster.Node.Registry.list_actor_pids/1","type":"function"},{"doc":"Get Process if this is alive.","ref":"Spawn.Cluster.Node.Registry.html#lookup/2","title":"Spawn.Cluster.Node.Registry.lookup/2","type":"function"},{"doc":"Update value meta inside registry","ref":"Spawn.Cluster.Node.Registry.html#update_entry_value/4","title":"Spawn.Cluster.Node.Registry.update_entry_value/4","type":"function"},{"doc":"Node subscriber","ref":"Spawn.Cluster.Node.Server.html","title":"Spawn.Cluster.Node.Server","type":"module"},{"doc":"","ref":"Spawn.Cluster.Node.Server.html#error/2","title":"Spawn.Cluster.Node.Server.error/2","type":"function"},{"doc":"","ref":"Spawn.Cluster.Node.Server.html#request/1","title":"Spawn.Cluster.Node.Server.request/1","type":"function"},{"doc":"Behavior for managing the state of clustered processes a.k.a lookups.","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour","type":"behaviour"},{"doc":"Cleanup action.","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#c:clean/2","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.clean/2","type":"callback"},{"doc":"Fetch the ActorHost process reference by id.\nIn case `id` is the hash of the ActorId but here the struct ActorId is passed as a parameter.\nAn implementations must handle this.","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#c:get_by_id/2","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.get_by_id/2","type":"callback"},{"doc":"Any procedure to be executed after the StateHandoffManager is initialized.\nExecuted during callback call to handle_continue.","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#c:handle_after_init/1","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.handle_after_init/1","type":"callback"},{"doc":"Any initialization code required by implementations of this behavior.\nMust return the state to be added in the StateHandoffManager.","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#c:handle_init/1","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.handle_init/1","type":"callback"},{"doc":"If necessary any procedure to be executed during a nodedown event","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#c:handle_nodedown_event/3","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.handle_nodedown_event/3","type":"callback"},{"doc":"If necessary any procedure to be executed during a nodeup event","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#c:handle_nodeup_event/3","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.handle_nodeup_event/3","type":"callback"},{"doc":"Perform any necessary cleanups during StateHandoffManager termination.\nGenerally excluding references to all processes owned by the terminating node.","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#c:handle_terminate/2","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.handle_terminate/2","type":"callback"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#c:handle_timer/2","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.handle_timer/2","type":"callback"},{"doc":"Adds a reference to an ActorHost process.","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#c:set/4","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.set/4","type":"callback"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#t:data/0","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.data/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#t:host/0","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.host/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#t:hosts/0","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.hosts/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#t:id/0","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.id/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#t:new_data/0","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.new_data/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#t:node_type/0","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.node_type/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#t:opts/0","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.opts/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.ControllerBehaviour.html#t:timer/0","title":"Spawn.Cluster.StateHandoff.ControllerBehaviour.timer/0","type":"type"},{"doc":"This handles state handoff in a cluster.\n\nIt uses the DeltaCrdt library to handle a distributed state, which is an eventually consistent replicated data type.\nThe module starts a GenServer that monitors nodes in the cluster, and when a new node comes up it sends a \"set_neighbours\"\nmessage to that node's GenServer process with its own DeltaCrdt process ID. This is done to ensure that changes in either node's\nstate are reflected across both.\n\nThe module also handles other messages like \"handoff\" and \"get\" to put and retrieve data from the DeltaCrdt state, respectively.","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController","type":"module"},{"doc":"Cluster HostActor cleanup","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#clean/2","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.clean/2","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#get_crdt_pid/0","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.get_crdt_pid/0","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#t:data/0","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.data/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#t:host/0","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.host/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#t:hosts/0","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.hosts/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#t:id/0","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.id/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#t:new_data/0","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.new_data/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#t:node_type/0","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.node_type/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#t:opts/0","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.opts/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.html#t:timer/0","title":"Spawn.Cluster.StateHandoff.Controllers.CrdtController.timer/0","type":"type"},{"doc":"`StateHandoffPersistentController` is a StateHandoff Controller basead on `Statestore` mechanism.","ref":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.html","title":"Spawn.Cluster.StateHandoff.Controllers.PersistentController","type":"module"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.html#t:data/0","title":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.data/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.html#t:host/0","title":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.host/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.html#t:hosts/0","title":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.hosts/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.html#t:id/0","title":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.id/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.html#t:new_data/0","title":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.new_data/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.html#t:node_type/0","title":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.node_type/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.html#t:opts/0","title":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.opts/0","type":"type"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.html#t:timer/0","title":"Spawn.Cluster.StateHandoff.Controllers.PersistentController.timer/0","type":"type"},{"doc":"This handles invocation scheduler stream\n\nIt uses the DeltaCrdt library to handle a distributed state, which is an eventually consistent replicated data type.\nThe module starts a GenServer that monitors nodes in the cluster, and when a new node comes up it sends a \"set_neighbours\"\nmessage to that node's GenServer process with its own DeltaCrdt process ID. This is done to ensure that changes in either node's\nstate are reflected across both.","ref":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.html","title":"Spawn.Cluster.StateHandoff.InvocationSchedulerState","type":"module"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.html#all/0","title":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.all/0","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.html#child_spec/1","title":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.child_spec/1","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.html#get/1","title":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.get/1","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.html#get_crdt_pid/0","title":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.get_crdt_pid/0","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.html#put/3","title":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.put/3","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.html#put_many/1","title":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.put_many/1","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.html#remove/1","title":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.remove/1","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.html#start_link/1","title":"Spawn.Cluster.StateHandoff.InvocationSchedulerState.start_link/1","type":"function"},{"doc":"This handles state handoff in a cluster.\n\nThis module monitors node up and down events as well as node terminate events and triggers `Spawn.Cluster.StateHandoff.ControllerBehaviour` implementations to handle these events.","ref":"Spawn.Cluster.StateHandoff.Manager.html","title":"Spawn.Cluster.StateHandoff.Manager","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Spawn.Cluster.StateHandoff.Manager.html#child_spec/1","title":"Spawn.Cluster.StateHandoff.Manager.child_spec/1","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Manager.html#child_spec/2","title":"Spawn.Cluster.StateHandoff.Manager.child_spec/2","type":"function"},{"doc":"Cluster HostActor cleanup","ref":"Spawn.Cluster.StateHandoff.Manager.html#clean/1","title":"Spawn.Cluster.StateHandoff.Manager.clean/1","type":"function"},{"doc":"Pickup the stored entity data for a actor","ref":"Spawn.Cluster.StateHandoff.Manager.html#get/1","title":"Spawn.Cluster.StateHandoff.Manager.get/1","type":"function"},{"doc":"Store a actor and entity in the lookup store","ref":"Spawn.Cluster.StateHandoff.Manager.html#set/2","title":"Spawn.Cluster.StateHandoff.Manager.set/2","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Manager.html#start_link/1","title":"Spawn.Cluster.StateHandoff.Manager.start_link/1","type":"function"},{"doc":"","ref":"Spawn.Cluster.StateHandoff.Manager.State.html","title":"Spawn.Cluster.StateHandoff.Manager.State","type":"module"},{"doc":"`AnySerializer` is responsible for serializing the protobuf type Any to Elixir\nstructures and vice versa.","ref":"Spawn.Utils.AnySerializer.html","title":"Spawn.Utils.AnySerializer","type":"module"},{"doc":"","ref":"Spawn.Utils.AnySerializer.html#any_pack!/1","title":"Spawn.Utils.AnySerializer.any_pack!/1","type":"function"},{"doc":"","ref":"Spawn.Utils.AnySerializer.html#any_unpack!/2","title":"Spawn.Utils.AnySerializer.any_unpack!/2","type":"function"},{"doc":"","ref":"Spawn.Utils.AnySerializer.html#json_any_pack!/1","title":"Spawn.Utils.AnySerializer.json_any_pack!/1","type":"function"},{"doc":"","ref":"Spawn.Utils.AnySerializer.html#pack_all_to_any/2","title":"Spawn.Utils.AnySerializer.pack_all_to_any/2","type":"function"},{"doc":"","ref":"Spawn.Utils.AnySerializer.html#unpack_any_bin/1","title":"Spawn.Utils.AnySerializer.unpack_any_bin/1","type":"function"},{"doc":"","ref":"Spawn.Utils.AnySerializer.html#unpack_unknown/1","title":"Spawn.Utils.AnySerializer.unpack_unknown/1","type":"function"},{"doc":"Error raised when the Actor already activated on another node.","ref":"Actors.Exceptions.NetworkPartitionException.html","title":"Actors.Exceptions.NetworkPartitionException","type":"exception"},{"doc":"","ref":"Actors.Exceptions.NetworkPartitionException.html#message/1","title":"Actors.Exceptions.NetworkPartitionException.message/1","type":"function"},{"doc":"Error raised when the Actor understands the request but refuses to authorize it.\n\nThis error should result in a http 403 status code.\nsee: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/403","ref":"Actors.Exceptions.NotAuthorizedException.html","title":"Actors.Exceptions.NotAuthorizedException","type":"exception"},{"doc":"","ref":"Actors.Exceptions.NotAuthorizedException.html#message/1","title":"Actors.Exceptions.NotAuthorizedException.message/1","type":"function"},{"doc":"# Spawn\n\n \n\n![Sepp](docs/images/sepp-elixir-254-400.png#gh-light-mode-only)\n![Sepp](docs/images/sepp-elixir-254-400.png#gh-dark-mode-only)\n\n**Actor Mesh Distributed Runtime**\n\n![ci](https://github.com/eigr/spawn/actions/workflows/ci.yaml/badge.svg)\n![last commit](https://img.shields.io/github/last-commit/eigr/spawn?style=social)\n[![join discord](https://badgen.net/badge/discord/Join%20Eigr%20on%20Discord/discord?icon=discord&label&color=blue)](https://discord.gg/2PcshvfS93)\n[![twitter](https://badgen.net/badge/twitter/@eigr_io/blue?label&icon=twitter)](https://twitter.com/eigr_io)\n\n### **[Website](https://eigr.io)** • **[Getting Started](docs/getting_started.md)** • **[SDKs](docs/sdks.md)** • **[Documentation](https://eigr.io/docs/projects-spawn/spawn-introduction/)** • **[Blog](https://eigr.io/blog/)**","ref":"readme.html","title":"Spawn","type":"extras"},{"doc":"Since UC Berkeley published its [Cloud Programming Simplified: A Berkeley View on\nServerless Computing](https://www2.eecs.berkeley.edu/Pubs/TechRpts/2019/EECS-2019-3.pdf) in the year of 2019, several solutions for Stateful Serverless and Durable Computing are emerging on the market.\nOriginally coming from some contributors to the [Cloudstate](https://github.com/cloudstateio/cloudstate), Spawn stands as the Erlang world's answer to the challenges of durable serverless computing. \n\nUnlike serverless runtimes hidden in cloud providers' black boxes, Spawn is an open-source serverless runtime designed for both cloud and on-premises environments. Powered by [BEAM](https://www.erlang.org/blog/a-brief-beam-primer/) and [ERTS](https://www.erlang.org/doc/apps/erts/), Spawn offers a polyglot programming model for crafting versatile applications. It's **BEAM** for all.\n\nExplore the potential of Erlang, regardless of your preferred programming language, to swiftly accomplish your business objectives. Check out our [documentation](docs/index.md) to [get started](docs/getting_started.md).","ref":"readme.html#overview","title":"Overview - Spawn","type":"extras"},{"doc":"> **_Tip:_** Serverless is not just FaaS\n\nServerless computing is a cloud computing execution model characterized by the automatic management of infrastructure, allowing developers to focus solely on code without the need to provision or manage servers explicitly. While commonly associated with cloud services, it's important to note that some serverless products operate independently of specific cloud providers, like Spawn. In these cases, the fundamental principles of automatic scaling, pay-per-execution pricing, and the abstraction of infrastructure complexities still apply, offering developers a serverless experience without exclusive reliance on a particular cloud platform.\n\nWhile serverless is often associated with Function as a Service (FaaS), where functions are the unit of deployment and execution, serverless computing can extend beyond just FaaS. In a broader sense, serverless includes services like backend-as-a-service (BaaS) and other cloud offerings where developers can build and deploy applications without dealing with the underlying infrastructure.\n\nThe key features of serverless computing include automatic scaling, pay-per-execution pricing, and a shift of operational responsibilities from the developer to the cloud provider or underlying runtime. Developers can focus more on writing code and implementing business logic, leaving the infrastructure management, scaling, and maintenance to the upper runtime.\n\nSpawn extends this Serverless computing model by considering that there is an infrastructure layer in all application code, such as state management, configuration management, integration flows with other applications, connection pooling, and so on, and that this code infrastructure must also be managed by the runtime, freeing the developer so that he can focus much more on his business objectives directly.\n\nThis also brings security to CEOs and CTOs as well as product managers that the central objectives of their companies will be achieved within a shorter period of time, thus increasing their competitiveness against their competitors. This without compromising technical quality and scalability to support your products running in production.","ref":"readme.html#demystifying-the-serverless-computing-model","title":"Demystifying the Serverless Computing Model - Spawn","type":"extras"},{"doc":"* https://eigr.io/blog/spawn-the-actor-mesh/\n\n* https://eigr.io/blog/beyond-monoliths-and-microservices/\n\n* https://eigr.io/blog/distributed-elixir-made-easy-with-spawn/","ref":"readme.html#latest-blogs","title":"Latest Blogs - Spawn","type":"extras"},{"doc":"Watch some insightful talks on Eigr Community and Spawn:\n\n- **Marcel Lanz on ACM SIGPLAN - Erlang 2021 - Lightning Talk**: https://www.youtube.com/watch?v=jVf0QqNb3qk _(English)_\n- **Marcel Lanz on Code Beam Europe 2022**: https://youtu.be/jgR7Oc_GXAg _(English)_\n- **Adriano Santos on Code Beam BR 2022**: https://www.youtube.com/watch?v=dXp0lyfmV_0&list=PLa5zLmv3pgCCiyWq8gltVAt2vTGuHz3eG&index=3 _(Portuguese)_\n- **Adriano Santos ElugSP 2023**: https://www.youtube.com/watch?v=MKTqiAtpK1E _(Portuguese)_\n- **Elias Arruda on Elixir CWB (Curitiba)**: https://www.youtube.com/live/yE_uWbPEWnI?si=5L3SORG3PERZpQ4V&t=463 _(Portuguese)_","ref":"readme.html#engaging-talks","title":"Engaging Talks - Spawn","type":"extras"},{"doc":"Say hello to Sepp, our cyberpunk mascot and proud [Ibex](https://alpshiking.swisshikingvacations.com/spotlight-on-the-ibex/). Sepp, hailing from the Swiss Alps, loves wandering through the Eiger mountains. Despite occasional Viking-like bluntness, he's a helpful companion who traces his lineage back to Viking times.\n\nThe name \"Sepp\" is indeed a German diminutive or nickname for the name Joseph. It's similar to how \"Joe\" is used as a nickname for Joseph in English. In German-speaking regions, it's common for Joseph to be affectionately referred to as \"Sepp.\"\n\nThis is our little tribute to one of the creators of the Erlang programming language, [Joe Armstrong](https://en.wikipedia.org/wiki/Joe_Armstrong_(programmer)).\n\n![Sepp Rules](docs/images/sepp-rules-254-400.png#gh-light-mode-only)\n![Sepp Rules](docs/images/sepp-rules-254-400.png#gh-dark-mode-only)\n\nWrite less code and be happy! 🌐🚀","ref":"readme.html#meet-sepp-our-cyberpunk-mascot","title":"Meet Sepp: Our Cyberpunk Mascot - Spawn","type":"extras"}]}